@@ -7,17 +7,16 @@ use Mojo::Log;
 use Mango::BSON ':bson';
 use MangoX::Queue::Delay;
 use MangoX::Queue::Job;
-use DateTime::Tiny;
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 # A logger
 has 'log' => sub { Mojo::Log->new->level('error') };
 
 # The Mango::Collection representing the queue
 has 'collection';
-has 'capped';
-has 'stats';
+has 'capped' => sub { $_[0]->stats->{capped} };
+has 'stats' => sub { $_[0]->collection->stats };
 
 # A MangoX::Queue::Delay
 has 'delay' => sub { MangoX::Queue::Delay->new };
@@ -45,13 +44,6 @@ sub new {
 
     croak qq{No Mango::Collection provided to constructor} unless ref($self->collection) eq 'Mango::Collection';
 
-    $self->stats($self->collection->stats);
-    $self->capped($self->stats->{capped});
-
-    $self->pending_status($self->capped ? 1 : 'Pending');
-    $self->processing_status($self->capped ? 2 : 'Processing');
-    $self->failed_status($self->capped ? 3 : 'Failed');
-
     return $self;
 }
 
@@ -80,6 +72,20 @@ sub plugin {
     return $self->plugins->{$name};
 }
 
+sub init_status {
+    my ($self) = @_;
+    # Done as late as possible - $self->capped opens a DB connection
+    $self->pending_status($self->capped ? 1 : 'Pending');
+    $self->processing_status($self->capped ? 2 : 'Processing');
+    $self->failed_status($self->capped ? 3 : 'Failed');
+    return {
+        pending_and_processing_status => $self->{pending_and_processing_status},
+        _pending_status => $self->{_pending_status},
+        _processing_status => $self->{_processing_status},
+        _failed_status => $self->{_failed_status},
+    };
+}
+
 sub pending_status {
     my ($self, $new_status) = @_;
 
@@ -88,6 +94,7 @@ sub pending_status {
     $self->{_pending_status} = $new_status;
     $self->_combine_pending_and_processing_status();
 }
+
 sub processing_status {
     my ($self, $new_status) = @_;
 
@@ -96,6 +103,7 @@ sub processing_status {
     $self->{_processing_status} = $new_status;
     $self->_combine_pending_and_processing_status();
 }
+
 sub failed_status {
     my ($self, $new_status) = @_;
 
@@ -118,7 +126,7 @@ sub get_options {
     return {
         query => {
             status => {
-                '$in' => $self->{pending_and_processing_status},
+                '$in' => $self->{pending_and_processing_status} // $self->init_status->{pending_and_processing_status},
             },
             processing => {
                 '$lt' => time - $self->timeout,
@@ -137,7 +145,7 @@ sub get_options {
         update => {
             '$set' => {
                 processing => time,
-                status => $self->{_processing_status},
+                status => $self->{_processing_status} // $self->init_status->{_processing_status},
             },
             '$inc' => {
                 attempt => 1,
@@ -162,9 +170,9 @@ sub enqueue {
 
     my $db_job = {
         priority => $args{priority} // 1,
-        created => $args{created} // DateTime::Tiny->now,
+        created => $args{created} // bson_time,
         data => $job,
-        status => $args{status} // $self->{_pending_status},
+        status => $args{status} // $self->{_pending_status} // $self->init_status->{_pending_status},
         attempt => 1,
         processing => 0,
         delay_until => 0,
@@ -258,7 +266,8 @@ sub _watch_nonblocking {
 sub requeue {
     my ($self, $job, $callback) = @_;
 
-    $job->{status} = ref($self->{_pending_status}) eq 'ARRAY' ? $self->{_pending_status}->[0] : $self->{_pending_status};
+    my $pending = $self->{_pending_status} // $self->init_status->{_pending_status};
+    $job->{status} = ref($pending) eq 'ARRAY' ? $pending->[0] : $pending;
     return $self->update($job, $callback);
 }
 
@@ -311,12 +320,8 @@ sub get {
 sub update {
     my ($self, $job, $callback) = @_;
 
-    # FIXME Temporary fix to remove queue item from MangoX::Queue::Job
-    my $j = {};
-    for my $key (keys %$job) {
-        $j->{$key} = $job->{$key} if $key ne 'queue';
-    }
-    $job = $j;
+    # FIXME Temporary fix to remove has_finished indicator from MangoX::Queue::Job
+    $job = { map { $_ => $job->{$_} } grep { $_ ne 'has_finished' } keys %$job };
 
     if($callback) {
         return $self->collection->update({'_id' => $job->{_id}}, $job => sub {
@@ -398,7 +403,7 @@ sub _consume_blocking {
         $self->log->debug("Job found by Mango: " . ($doc ? 'Yes' : 'No'));
 
         if($doc && $doc->{attempt} > $self->retries) {
-            $doc->{status} = $self->{_failed_status};
+            $doc->{status} = $self->{_failed_status} // $self->init_status->{_failed_status};
             $self->update($doc);
             $doc = undef;
             $self->log->debug("Job exceeded retries, status set to failed and job abandoned");
@@ -450,7 +455,7 @@ sub _consume_nonblocking {
         }
         
         if($doc && $doc->{attempt} > $self->retries) {
-            $doc->{status} = $self->{_failed_status};
+            $doc->{status} = $self->{_failed_status} // $self->init_status->{_failed_status};
             $self->update($doc);
             $doc = undef;
             $self->log->debug("Job exceeded retries, status set to failed and job abandoned");
@@ -461,7 +466,10 @@ sub _consume_nonblocking {
             $self->log->debug("job_count incremented to " . $self->job_count);
 
             my $job = MangoX::Queue::Job->new($doc);
-            $job->queue($self);
+            $job->once(finished => sub {
+                $self->job_count($self->job_count - 1);
+                $self->log->debug('job_count decremented to ' . $self->job_count);
+            });
 
             $self->delay->reset;
             $self->emit_safe(consumed => $job) if $self->has_subscribers('consumed');
@@ -521,7 +529,7 @@ Non-blocking mode requires a running L<Mojo::IOLoop>.
     enqueue $queue 'test' => sub { my $id = shift; };
 
     # To set options
-    enqueue $queue priority => 1, created => DateTime::Tiny->now, 'test' => sub { my $id = shift; };
+    enqueue $queue priority => 1, created => bson_time, 'test' => sub { my $id = shift; };
 
     # To watch for a specific job status
     watch $queue $id, 'Complete' => sub {
@@ -563,7 +571,7 @@ Non-blocking mode requires a running L<Mojo::IOLoop>.
     my $id = enqueue $queue 'test';
 
     # To set options
-    my $id = enqueue $queue priority => 1, created => DateTime::Tiny->now, 'test';
+    my $id = enqueue $queue priority => 1, created => bson_time, 'test';
 
     # To watch for a specific job status
     watch $queue $id;
@@ -633,6 +641,11 @@ the internal counter. See L<MangoX::Queue::Job> for more details.
 Set to -1 to disable queue concurrency limits. B<Use with caution>, this could result in
 out of memory errors or an extremely slow event loop.
 
+If you need to decrement the job counter early (e.g. to hold on to a reference to the job after you've
+finished processing it), you can call the C<finished> method on the L<MangoX::Queue::Job> object.
+
+    $job->finished;
+
 =head2 failed_status
 
     $self->failed_status('Failed');
@@ -757,7 +770,7 @@ You can set queue options including priority, created and status.
 
     my $id = enqueue $queue,  
         priority => 1,
-        created => time,
+        created => bson_time,
         status => 'Pending',
         +{
             foo => 'bar'
